import { createEventHandler } from "./events";
export function createCatcher(eventsSpec) {
    const n = Object.keys(eventsSpec).length;
    class VCatcher {
        constructor(child, handlers) {
            this.handlerFns = [];
            this.afterNode = null;
            this.child = child;
            this.handlerData = handlers;
        }
        mount(parent, afterNode) {
            this.parentEl = parent;
            this.child.mount(parent, afterNode);
            this.afterNode = document.createTextNode("");
            parent.insertBefore(this.afterNode, afterNode);
            this.wrapHandlerData();
            for (let name in eventsSpec) {
                const index = eventsSpec[name];
                const handler = createEventHandler(name);
                this.handlerFns[index] = handler;
                handler.setup.call(parent, this.handlerData[index]);
            }
        }
        wrapHandlerData() {
            for (let i = 0; i < n; i++) {
                let handler = this.handlerData[i];
                // handler = [...mods, fn, comp], so we need to replace second to last elem
                let idx = handler.length - 2;
                let origFn = handler[idx];
                const self = this;
                handler[idx] = function (ev) {
                    const target = ev.target;
                    let currentNode = self.child.firstNode();
                    const afterNode = self.afterNode;
                    while (currentNode && currentNode !== afterNode) {
                        if (currentNode.contains(target)) {
                            return origFn.call(this, ev);
                        }
                        currentNode = currentNode.nextSibling;
                    }
                };
            }
        }
        moveBeforeDOMNode(node, parent = this.parentEl) {
            this.parentEl = parent;
            this.child.moveBeforeDOMNode(node, parent);
            parent.insertBefore(this.afterNode, node);
        }
        moveBeforeVNode(other, afterNode) {
            if (other) {
                // check this with @ged-odoo for use in foreach
                afterNode = other.firstNode() || afterNode;
            }
            this.child.moveBeforeVNode(other ? other.child : null, afterNode);
            this.parentEl.insertBefore(this.afterNode, afterNode);
        }
        patch(other, withBeforeRemove) {
            if (this === other) {
                return;
            }
            this.handlerData = other.handlerData;
            this.wrapHandlerData();
            for (let i = 0; i < n; i++) {
                this.handlerFns[i].update.call(this.parentEl, this.handlerData[i]);
            }
            this.child.patch(other.child, withBeforeRemove);
        }
        beforeRemove() {
            this.child.beforeRemove();
        }
        remove() {
            for (let i = 0; i < n; i++) {
                this.handlerFns[i].remove.call(this.parentEl);
            }
            this.child.remove();
            this.afterNode.remove();
        }
        firstNode() {
            return this.child.firstNode();
        }
        toString() {
            return this.child.toString();
        }
    }
    return function (child, handlers) {
        return new VCatcher(child, handlers);
    };
}
