import { config } from "./config";
export function createEventHandler(rawEvent) {
    const eventName = rawEvent.split(".")[0];
    const capture = rawEvent.includes(".capture");
    if (rawEvent.includes(".synthetic")) {
        return createSyntheticHandler(eventName, capture);
    }
    else {
        return createElementHandler(eventName, capture);
    }
}
// Native listener
let nextNativeEventId = 1;
function createElementHandler(evName, capture = false) {
    let eventKey = `__event__${evName}_${nextNativeEventId++}`;
    if (capture) {
        eventKey = `${eventKey}_capture`;
    }
    function listener(ev) {
        const currentTarget = ev.currentTarget;
        if (!currentTarget || !document.contains(currentTarget))
            return;
        const data = currentTarget[eventKey];
        if (!data)
            return;
        config.mainEventHandler(data, ev, currentTarget);
    }
    function setup(data) {
        this[eventKey] = data;
        this.addEventListener(evName, listener, { capture });
    }
    function remove() {
        delete this[eventKey];
        this.removeEventListener(evName, listener, { capture });
    }
    function update(data) {
        this[eventKey] = data;
    }
    return { setup, update, remove };
}
// Synthetic handler: a form of event delegation that allows placing only one
// listener per event type.
let nextSyntheticEventId = 1;
function createSyntheticHandler(evName, capture = false) {
    let eventKey = `__event__synthetic_${evName}`;
    if (capture) {
        eventKey = `${eventKey}_capture`;
    }
    setupSyntheticEvent(evName, eventKey, capture);
    const currentId = nextSyntheticEventId++;
    function setup(data) {
        const _data = this[eventKey] || {};
        _data[currentId] = data;
        this[eventKey] = _data;
    }
    function remove() {
        delete this[eventKey];
    }
    return { setup, update: setup, remove };
}
function nativeToSyntheticEvent(eventKey, event) {
    let dom = event.target;
    while (dom !== null) {
        const _data = dom[eventKey];
        if (_data) {
            for (const data of Object.values(_data)) {
                const stopped = config.mainEventHandler(data, event, dom);
                if (stopped)
                    return;
            }
        }
        dom = dom.parentNode;
    }
}
const CONFIGURED_SYNTHETIC_EVENTS = {};
function setupSyntheticEvent(evName, eventKey, capture = false) {
    if (CONFIGURED_SYNTHETIC_EVENTS[eventKey]) {
        return;
    }
    document.addEventListener(evName, (event) => nativeToSyntheticEvent(eventKey, event), {
        capture,
    });
    CONFIGURED_SYNTHETIC_EVENTS[eventKey] = true;
}
